# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from ebi_eva_common_pyutils.network_utils import json_request


def get_scientific_name_from_ensembl(taxonomy_id: int) -> str:
    ENSEMBL_REST_API_URL = "https://rest.ensembl.org/taxonomy/id/{0}?content-type=application/json".format(taxonomy_id)
    response = json_request(ENSEMBL_REST_API_URL)
    if "scientific_name" not in response:
        raise Exception("Scientific name could not be found for taxonomy {0} using the Ensembl API URL: {1}"
                        .format(taxonomy_id, ENSEMBL_REST_API_URL))
    return response["scientific_name"]


def get_normalized_scientific_name_from_ensembl(taxonomy_id: int) -> str:
    # Match Ensembl representation
    # See Clostridium sp. SS2/1 represented as clostridium_sp_ss2_1 in
    # ftp://ftp.ensemblgenomes.org/pub/bacteria/release-48/fasta/bacteria_25_collection/clostridium_sp_ss2_1/
    return re.sub('[^0-9a-zA-Z]+', '_', get_scientific_name_from_ensembl(taxonomy_id).lower())



# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib.parse import quote_plus
from lxml import etree as et
import json
import yaml
import urllib.request
from retry import retry


class EVAPrivateSettingsXMLConfig:
    config_data = None

    def __init__(self, settings_xml_file: str):
        with open(settings_xml_file) as xml_file_handle:
            self.config_data = et.parse(xml_file_handle)

    def get_value_with_xpath(self, location: str):
        etree = self.config_data.getroot()
        result = etree.xpath(location)
        if not result:
            raise ValueError("Invalid XPath location: " + location)
        return result


def get_pg_uri_for_accession_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.accession.jdbc.url")


def get_pg_uri_for_variant_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.variant.jdbc.url")


def get_pg_metadata_uri_for_eva_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.evapro.jdbc.url")


def get_pg_uri_details_for_profile(eva_profile_name: str, settings_xml_file: str, tag_name: str):
    config = EVAPrivateSettingsXMLConfig(settings_xml_file)
    xpath_location_template = '//settings/profiles/profile/id[text()="{0}"]/../properties/{1}/text()'
    # Format is jdbc:postgresql://host:port/db
    metadata_db_jdbc_url = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, tag_name))[0]
    return metadata_db_jdbc_url.split("jdbc:")[-1]


def get_mongo_uri_for_eva_profile(eva_profile_name: str, settings_xml_file: str):
    config = EVAPrivateSettingsXMLConfig(settings_xml_file)
    xpath_location_template = '//settings/profiles/profile/id[text()="{0}"]/../properties/{1}/text()'
    # Format is host1:port1,host2:port2
    mongo_hosts_and_ports = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.host"))[0]
    username = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.user"))[0]
    password = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.passwd"))[0]
    authentication_db = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.auth.db"))[0]
    return "mongodb://{0}:{1}@{2}/{3}".format(username, quote_plus(password), mongo_hosts_and_ports, authentication_db)


def get_properties_from_xml_file(profile, xml_path):
    tree = et.parse(xml_path)
    root = tree.getroot()
    return get_profile_properties(profile, root)


def get_properties_from_xml_string(profile, str):
    root = et.fromstring(str)
    return get_profile_properties(profile, root)


def get_profile_properties(profile, root):
    properties = {}
    for property in root.xpath('//settings/profiles/profile/id[text()="' + profile + '"]/../properties/*'):
        properties[property.tag] = property.text
    return properties


@retry(tries=4, delay=2, backoff=1.2, jitter=(1, 3))
def get_eva_settings_xml_string(token):
    url = 'https://api.github.com/repos/EBIvariation/configuration/contents/eva-maven-settings.xml'
    headers = {'Authorization': 'token ' + token, 'Accept' : 'application/vnd.github.raw' }
    request = urllib.request.Request(url, None, headers)
    with urllib.request.urlopen(request) as response:
        return response.read()


def get_args_from_private_config_file(private_config_file):
    with open(private_config_file) as private_config_file_handle:
        if 'json' in private_config_file:
            return json.load(private_config_file_handle)
        else:
            if 'yml' in private_config_file:
                return yaml.safe_load(private_config_file_handle)
            else:
                raise TypeError('Configuration file should be either json or yaml')


import os

import yaml


class Configuration:
    """
    Configuration class that allow to load a yaml file either at construction or later in the execution.
    It can be used like a dict but should be used as readonly.
    """
    config_file = None
    content = {}

    def __init__(self, *search_path):
        if search_path:
            self.load_config_file(*search_path)

    @staticmethod
    def _find_config_file(search_path):
        for p in search_path:
            if p and os.path.isfile(p):
                return p

    def load_config_file(self, *search_path):
        self.config_file = self._find_config_file(search_path)
        if self.config_file:
            with open(self.config_file, 'r') as f:
                self.content = yaml.safe_load(f)
        else:
            raise FileNotFoundError('Could not find any config file in specified search path')

    def get(self, item, ret_default=None):
        """
        Dict-style item retrieval with default
        :param item: The key to search for
        :param ret_default: What to return if the key is not present
        """
        try:
            return self[item]
        except KeyError:
            return ret_default

    def query(self, *parts, ret_default=None):
        """
        Drill down into a config, e.g. cfg.query('logging', 'handlers', 'a_handler', 'level')
        :param ret_default:
        :return: The relevant item if it exists in the config, else ret_default.
        """
        top_level = self.content
        item = None

        for p in parts:
            item = top_level.get(p)
            if item:
                top_level = item
            else:
                return ret_default
        return item

    def report(self):
        return yaml.safe_dump(self.content, default_flow_style=False)

    def __getitem__(self, item):
        """Allow dict-style access, e.g. config['this'] or config['this']['that']."""
        return self.content[item]

    def __contains__(self, item):
        """Allow search in the first layer of the config with 'in' operator."""
        return self.content.__contains__(item)


cfg = Configuration()
"""
Provides a singleton that can be used as a central place for configuration.
"""



# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import requests
import subprocess
from retry import retry

from ebi_eva_common_pyutils.logger import logging_config as log_cfg

logger = log_cfg.get_logger(__name__)


def is_port_in_use(port):
    import socket
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('localhost', port)) == 0


def get_available_local_port(try_starting_with_port):
    for i in range(0, 20):
        port_to_try = try_starting_with_port + i
        logger.info("Attempting to forward remote mongo port to local port {0}...".format(port_to_try))
        if is_port_in_use(port_to_try):
            logger.info("Port {0} already in use...".format(port_to_try))
        else:
            return port_to_try
    logger.error("Could not forward to any local port!")


def forward_remote_port_to_local_port(remote_host: str, remote_port: int, local_port: int) -> int:
    port_forward_command = 'ssh -N -L{0}:localhost:{1} {2}'.format(local_port, remote_port, remote_host)
    logger.info("Forwarding port to local port using command: " + port_forward_command)
    proc = subprocess.Popen(port_forward_command.split(" "))
    return proc.pid


@retry(exceptions=(ConnectionError, requests.RequestException), logger=logger,
       tries=4, delay=2, backoff=1.2, jitter=(1, 3))
def json_request(url: str, payload: dict = None, method=requests.get) -> dict:
    """Makes a request of a specified type (by default GET) with the specified URL and payload, attempts to parse the
    result as a JSON string and return it as a dictionary, on failure raises an exception."""
    result = method(url, data=payload)
    result.raise_for_status()
    return result.json()




# Copyright 2021 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import subprocess

from cached_property import cached_property
from pymongo import MongoClient, uri_parser

from ebi_eva_common_pyutils.command_utils import run_command_with_output
from ebi_eva_common_pyutils.logger import AppLogger


class MongoDatabase(AppLogger):
    def __init__(self, uri: str, secrets_file: str = None, db_name: str = "admin"):
        self.uri = uri
        self.secrets_file = secrets_file
        self.db_name = db_name

    @cached_property
    def uri_with_db_name(self):
        """
        Return URI with the database name substituted
        ex:
        If the URI is mongodb://user@localhost:27017/admin and database name is eva_fcatus_90,
        then the URI with the database name will be mongodb://user@localhost:27017/eva_fcatus_90?authSource=admin
        """
        if self.db_name == "admin":
            return self.uri
        uri_components = uri_parser.parse_uri(self.uri)
        username_component = f"{uri_components['username']}@" if uri_components['username'] else ""
        # Hack needed to log in to a different DB but retain authentication source
        # See https://docs.mongodb.com/v4.0/reference/connection-string/#records-database and https://docs.mongodb.com/v4.0/reference/connection-string/#urioption.authSource
        uri_with_db_name = f"mongodb://{username_component}" + \
                           ",".join([node + ':' + str(port) for node, port in uri_components['nodelist']]) + \
                           f"/{self.db_name}"
        uri_with_db_name += f"?authSource=" \
                            f"{uri_components['options'].get('authSource', 'admin')}" if self.secrets_file else ""
        return uri_with_db_name

    @cached_property
    def mongo_handle(self):
        if self.secrets_file:
            with open(self.secrets_file) as secrets_file_handle:
                mongo_password = secrets_file_handle.read().strip()
            return MongoClient(self.uri, password=mongo_password)
        else:
            return MongoClient(self.uri)

    def __del__(self):
        self.mongo_handle.close()

    def _get_optional_secrets_file_stdin(self):
        # Mongodump and restore tools are notorious in displaying clear text passwords
        # in commands - See https://jira.mongodb.org/browse/TOOLS-1020
        # A secrets file can be provided to work around it
        if self.secrets_file:
            return f" < {self.secrets_file}"
        return ""

    def drop(self):
        self.mongo_handle.drop_database(self.db_name)

    def get_collection_names(self):
        return self.mongo_handle[self.db_name].list_collection_names()

    def get_indexes(self):
        collection_index_map = {}
        for collection_name in self.get_collection_names():
            collection_index_map[collection_name] = self.mongo_handle[self.db_name][collection_name].index_information()
        return collection_index_map

    def create_index_on_collections(self, collection_index_map):
        collection_index_map_copy = copy.deepcopy(collection_index_map)
        for collection_name, index_info_map in collection_index_map_copy.items():
            # Copy indexes from one collection to another See https://stackoverflow.com/a/51445278
            for name, index_info in index_info_map.items():
                index_keys = index_info['key']
                del (index_info['ns'])
                del (index_info['v'])
                del (index_info['key'])
                if 'background' in index_info:
                    del (index_info['background'])
                # Due to https://jira.mongodb.org/browse/SERVER-11064
                # pre-v3.2 index sort indicators could allow for floats like 1.0
                # ex: (_id, 1.0) for ascending index on _id or (_id, -1.0) for descending index on _id
                # Since validation is stricter in database versions newer than v3.3.1, cast sort indicators to int
                for i, _ in enumerate(index_keys):
                    index_keys[i] = (index_keys[i][0], int(index_keys[i][1]))
                self.mongo_handle[self.db_name][collection_name].create_index(index_keys, name=name, **index_info)

    def enable_sharding(self):
        self.mongo_handle.admin.command({"enableSharding": self.db_name})

    def shard_collections(self, collections_shard_key_map, collections_to_shard):
        for collection_name in collections_to_shard:
            shard_key, shard_key_uniqueness_flag = collections_shard_key_map.get(collection_name, (["_id"], True))
            # Shard key representation in the format {"key1": 1, "key2": 1}
            shard_key_repr = "{{{0}}}".format(",".join([f'"{attribute}": 1' for attribute in shard_key]))
            shard_collection_command = f'sh.shardCollection(' \
                                       f'"{self.db_name}.{collection_name}", ' \
                                       f'{shard_key_repr}, {str(shard_key_uniqueness_flag).lower()})'
            sharding_command = f"mongo --host {self.uri}  " \
                               f"--eval " \
                               f"'{shard_collection_command}' "
            sharding_command += self._get_optional_secrets_file_stdin()
            run_command_with_output(f"Sharding collection {collection_name} in the database {self.uri_with_db_name} "
                                    f"with key {shard_key_repr}...", sharding_command,
                                    log_error_stream_to_output=True, )

    def dump_data(self, dump_dir, mongodump_args=None):
        mongodump_args = " ".join([f"--{arg} {val}"
                                   for arg, val in mongodump_args.items()]) if mongodump_args else ""
        mongodump_command = f"mongodump --uri {self.uri_with_db_name}  --out {dump_dir} {mongodump_args}" + \
                            self._get_optional_secrets_file_stdin()
        try:
            run_command_with_output("mongodump", mongodump_command, log_error_stream_to_output=True)
        except subprocess.CalledProcessError as ex:
            raise Exception("mongodump failed! HINT: Did you forget to provide a secrets file for authentication?")

    def archive_data(self, archive_dir, archive_name="archive", mongodump_args=None):
        mongodump_args = " ".join([f"--{arg} {val}"
                                   for arg, val in mongodump_args.items()]) if mongodump_args else ""
        mongodump_command = f"mongodump --uri {self.uri_with_db_name}  --archive={archive_dir}/{archive_name} {mongodump_args}" + \
                            self._get_optional_secrets_file_stdin()
        try:
            run_command_with_output("mongodump", mongodump_command, log_error_stream_to_output=True)
        except subprocess.CalledProcessError as ex:
            raise Exception("mongodump failed! HINT: Did you forget to provide a secrets file for authentication?")

    def restore_data(self, dump_dir, mongorestore_args=None):
        mongorestore_args = " ".join([f"--{arg} {val}"
                                      for arg, val in mongorestore_args.items()]) if mongorestore_args else ""
        mongorestore_command = f"mongorestore --uri {self.uri_with_db_name} " \
                               f"{mongorestore_args} " \
                               f"--dir {dump_dir} "
        mongorestore_command += self._get_optional_secrets_file_stdin()
        try:
            run_command_with_output("mongorestore", mongorestore_command, log_error_stream_to_output=True)
        except subprocess.CalledProcessError as ex:
            raise Exception("mongorestore failed! HINT: Did you forget to provide a secrets file for authentication?")

    def export_data(self, export_directory, mongoexport_args=None):
        mongoexport_args = " ".join([f"--{arg} {val}"
                                     for arg, val in mongoexport_args.items()]) if mongoexport_args else ""
        mongoexport_command = f"mongoexport --uri {self.uri_with_db_name}  --out {export_directory} {mongoexport_args}" + \
                              self._get_optional_secrets_file_stdin()
        try:
            run_command_with_output("mongoexport", mongoexport_command, log_error_stream_to_output=True)
        except subprocess.CalledProcessError as ex:
            raise Exception("mongoexport failed! HINT: Did you forget to provide a secrets file for authentication?")

    def import_data(self, coll_file_loc, mongoimport_args=None):
        mongoimport_args = " ".join([f"--{arg} {val}"
                                     for arg, val in mongoimport_args.items()]) if mongoimport_args else ""
        mongoimport_command = f"mongoimport --uri {self.uri_with_db_name}  --file {coll_file_loc} {mongoimport_args}" + \
                              self._get_optional_secrets_file_stdin()
        try:
            run_command_with_output("mongoimport", mongoimport_command, log_error_stream_to_output=True)
        except subprocess.CalledProcessError as ex:
            raise Exception("mongoexport failed! HINT: Did you forget to provide a secrets file for authentication?")



# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import psycopg2
from ebi_eva_common_pyutils.pg_utils import get_result_cursor, get_all_results_for_query


def get_db_conn_for_species(species_db_info):
    db_name = "dbsnp_{0}".format(species_db_info["dbsnp_build"])
    pg_conn = psycopg2.connect("dbname='{0}' user='{1}' host='{2}'  port={3}".
                               format(db_name, "dbsnp", species_db_info["pg_host"], species_db_info["pg_port"]))
    return pg_conn


def get_species_info(metadata_connection_handle, dbsnp_species_name="all"):
    get_species_info_query = "select distinct database_name, scientific_name, dbsnp_build, pg_host, pg_port from " \
                             "dbsnp_ensembl_species.import_progress a " \
                             "join dbsnp_ensembl_species.dbsnp_build_instance b " \
                                "on b.dbsnp_build = a.ebi_pg_dbsnp_build "
    if dbsnp_species_name != "all":
        get_species_info_query += "where database_name = '{0}' ".format(dbsnp_species_name)
    get_species_info_query += "order by database_name"

    pg_cursor = get_result_cursor(metadata_connection_handle, get_species_info_query)
    species_set = [{"database_name": result[0], "scientific_name": result[1], "dbsnp_build":result[2],
                    "pg_host":result[3], "pg_port":result[4]}
                   for result in pg_cursor.fetchall()]
    pg_cursor.close()
    return species_set


# Get connection information for each Postgres instance of the dbSNP mirror
def get_dbsnp_mirror_db_info(pg_metadata_dbname, pg_metadata_user, pg_metadata_host):
    with psycopg2.connect("dbname='{0}' user='{1}' host='{2}'".format(pg_metadata_dbname, pg_metadata_user,
                                                                      pg_metadata_host)) as pg_conn:
        dbsnp_mirror_db_info_query = "select * from dbsnp_ensembl_species.dbsnp_build_instance"
        dbsnp_mirror_db_info = [{"dbsnp_build": result[0], "pg_host": result[1], "pg_port": result[2]}
                                for result in get_all_results_for_query(pg_conn, dbsnp_mirror_db_info_query)]
    return dbsnp_mirror_db_info


def get_variant_warehouse_db_name_from_assembly_and_taxonomy(metadata_connection_handle, assembly, taxonomy):
    query = f"select t.taxonomy_code, a.assembly_code " \
            f"from assembly a " \
            f"left join taxonomy t on (t.taxonomy_id = a.taxonomy_id) " \
            f"where a.assembly_accession = '{assembly}'" \
            f"and a.taxonomy_id = {taxonomy}"
    rows = get_all_results_for_query(metadata_connection_handle, query)
    if len(rows) == 0:
        return None
    elif len(rows) > 1:
        options = ', '.join((f'{r[0]}_{r[1]}' for r in rows))
        raise ValueError(f'More than one possible database for assembly {assembly} and taxonomy {taxonomy} found: '
                         f'{options}')
    database_name = f'eva_{rows[0][0]}_{rows[0][1]}'
    return database_name



# Copyright 2019 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import urllib
from csv import DictReader, excel_tab
from ftplib import FTP
import re
from urllib import request

from cached_property import cached_property
from retry import retry

from ebi_eva_common_pyutils.command_utils import run_command_with_output
from ebi_eva_common_pyutils.logger import AppLogger


class NCBISequence(AppLogger):
    """
    Class that represent an Sequence that would originate from NCBI data
    It takes a any Genbank or refseq accession and can download the sequence genomics fasta form.
    Using species_scientific_name and assembly_accession it create a directory structure in the provided
    reference_directory:
        - species_scientific_name1
            - assembly_accession1
            - assembly_accession2
        - species_scientific_name2
    """

    insdc_accession_formats = [
        r'[A-Z][0-9]{5}\.[0-9]+',
        r'[A-Z]{2}[0-9]{6}\.[0-9]+',
        r'[A-Z]{2}[0-9]{8}\.[0-9]+',
        r'[A-Z]{4}[0-9]{8}\.[0-9]+',
        r'[A-Z]{6}[0-9]{9}\.[0-9]+'
    ]

    def __init__(self, sequence_accession, species_scientific_name, reference_directory, eutils_api_key=None):
        self.sequence_accession = sequence_accession
        self.species_scientific_name = species_scientific_name
        self.reference_directory = reference_directory
        self.eutils_api_key = eutils_api_key

    @staticmethod
    def is_genbank_accession_format(accession):
        if any(
                re.match(insdc_accession_format, accession)
                for insdc_accession_format in NCBISequence.insdc_accession_formats
        ):
            return True
        return False

    @staticmethod
    def check_genbank_accession_format(accession):
        if not NCBISequence.is_genbank_accession_format(accession):
            raise ValueError('Invalid INSDC accession: %s' % accession)

    @property
    def sequence_directory(self):
        sequence_directory = os.path.join(
            self.reference_directory,  self.species_scientific_name.lower().replace(' ', '_'), self.sequence_accession
        )
        os.makedirs(sequence_directory, exist_ok=True),
        return sequence_directory

    @property
    def sequence_fasta_path(self):
        return os.path.join(self.sequence_directory, self.sequence_accession + '.fa')

    def download_contig_sequence_from_ncbi(self, genbank_only=True):
        if genbank_only:
            self.check_genbank_accession_format(self.sequence_accession)
        self._download_contig_from_ncbi(self.sequence_accession, self.sequence_fasta_path)
        self.info(self.sequence_fasta_path + " downloaded and added to FASTA sequence")

    @retry(tries=4, delay=2, backoff=1.2, jitter=(1, 3))
    def _download_contig_from_ncbi(self, contig_accession, output_file):
        parameters = {
            'db': 'nuccore',
            'id': contig_accession,
            'rettype': 'fasta',
            'retmode': 'text',
            'tool': 'eva',
            'email': 'eva-dev@ebi.ac.uk'
        }
        if self.eutils_api_key:
            parameters['api_key'] = self.eutils_api_key
        url = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?' + urllib.parse.urlencode(parameters)
        self.info('Downloading ' + contig_accession)
        urllib.request.urlretrieve(url, output_file)

